import PropTypes from 'prop-types';
import { withRouter } from 'react-router-dom';
import React, { useCallback } from 'react';

let wtSmart;
if (process.env.NODE_ENV === 'development') {
    wtSmart = require('webtrekk-smart-pixel/debug');
}
else {
    wtSmart = require('webtrekk-smart-pixel');
}

/**
 * @type {wtSmart|null}
 */
var pixel_ = null;

/**
 * @returns {Window}
 */
const getWindow_ = function() {
    return window;
};

/**
 * @returns {HTMLDocument}
 */
const getDocument_ = function() {
    return window.document;
};

const emptyObject = {};

/**
 *
 */
const init_ = function() {
    const window_ = getWindow_();
    pixel_ = wtSmart.use(window_, getDocument_());
    window_['wtSmart'] = pixel_;
};

/**
 * @constructor
 */
class WebtrekkSmartPixelReact {
    /**
     * @param {function(wtSmart: wtSmart)} call
     */
    call(call) {
        if (pixel_ === null) {
            init_();
        }

        pixel_.push(call);
    };

    /**
     * @param {object} data
     */
    init(data = emptyObject) {
        this.call(function(pix) {
            pix.init.add(data);
        });
    };

    /**
     * @param {object} data
     */
    advanced(data = emptyObject) {
        this.call(function(pix) {
            pix.advanced.add(data);
        });
    };

    /**
     * @param {string} name
     * @param {object} data
     */
    page(name = '', data = emptyObject) {
        this.call(function(pix) {
            pix.page.data.add(name, data);
        });
    };

    /**
     * @param {object} data
     */
    action(data = emptyObject) {
        this.call(function(pix) {
            pix.action.data.add(data);
        });
    };

    /**
     * @param {object} data
     */
    session(data = emptyObject) {
        this.call(function(pix) {
            pix.session.data.add(data);
        });
    };

    /**
     * @param {object} data
     */
    campaign(data = emptyObject) {
        this.call(function(pix) {
            pix.campaign.data.add(data);
        });
    };

    /**
     * @param {string} id
     * @param {object} data
     * @param {boolean} validation
     */
    customer(id = '', data = emptyObject, validation = false) {
        this.call(function(pix) {
            pix.customer.data.add(id, data, validation);
        });
    };

    /**
     * @param {string} action
     * @param {object} data
     */
    product(action = 'view', data = emptyObject) {
        this.call(function(pix) {
            var method = (action === 'view' || action === 'basket') ? 'set' : 'add';
            pix.product[action].data[method]([data]);
        });
    };

    /**
     * @param {string} action
     * @param {Array} data
     */
    products(action = 'view', data = []) {
        this.call(function(pix) {
            var method = (action === 'view' || action === 'basket') ? 'set' : 'add';
            pix.product[action].data[method](data);
        });
    };

    /**
     * @param {object} data
     */
    order(data = emptyObject) {
        this.call(function(pix) {
            pix.order.data.add(data);
        });
    };

    /**
     * @param {string} extension
     * @param {string} action
     * @param {object} config
     */
    extension(extension = '', action = 'activate', config = emptyObject) {
        if (!extension) {
            return 0;
        }

        this.call(function(pix) {
            pix.extension[extension][action](config);
        });
    };

    /**
     * @param {boolean} keepData
     */
    track(keepData = false) {
        this.call(function(pix) {
            pix.track(keepData);
        });
    };

    /**
     *
     */
    trackPage(keepData = false) {
        this.call(function(pix) {
            pix.trackPage(keepData);
        });
    };

    /**
     *
     */
    trackAction(keepData = false) {
        this.call(function(pix) {
            pix.trackAction(keepData);
        });
    };
}

var SmartPixelReact = new WebtrekkSmartPixelReact();

class WebtrekkReactComponent extends React.Component {
    /**
     * @param {string} type
     */
    constructor(type) {
        super();
        this.type = type;
    }

    /**
     * @param {string} type
     * @param {object} props
     */
    addConfigurationData(type = this.type, props = this.props) {
        SmartPixelReact[type](props);
    }

    /**
     * @param {string} type
     * @param {object} props
     */
    addTrackingData(type = this.type, props = this.props) {
        SmartPixelReact[type](props);
    }

    /**
     * @param {object} props
     */
    addPageTrackingData(props = this.props) {
        SmartPixelReact.page(props.name, props);
    }

    /**
     * @param {object} props
     */
    addCustomerTrackingData(props = this.props) {
        SmartPixelReact.customer(props.id, props, props.validation);
    }

    /**
     * @param {object} props
     */
    addProductTrackingData(props = this.props) {
        SmartPixelReact.product(props.action, props);
    }

    /**
     * @param {string} extension
     * @param {string} action
     * @param {object} config
     */
    addExtension(extension, action = 'activate', config = {}) {
        SmartPixelReact.extension(extension, action, config);
    }

    /**
     * @param {object} ref
     * @param {object} props
     */
    addContentEngagementElement(ref = this.elementRef, props = this.props) {
        SmartPixelReact.call(function(pix) {
            pix.extension.content_engagement.activate();
            pix.extension.content_engagement.add({
                selector: ((props.selector) ? props.selector : ref.current),
                name: props.name,
                config: props
            });
        });
    }

    /**
     * @param {object} ref
     * @param {object} props
     */
    addProductListElement(ref = this.elementRef, props = this.props) {
        SmartPixelReact.call(function(pix) {
            pix.extension.product_list_tracking.activate();
            pix.extension.product_list_tracking.add({
                selector: ((props.selector) ? props.selector : ref.current),
                data: props
            });
        });
    }

    /**
     * @param {object} ref
     * @param {object} props
     */
    addTeaserElement(ref = this.elementRef, props = this.props) {
        SmartPixelReact.call(function(pix) {
            pix.extension.teaser_tracking.activate();
            pix.extension.teaser_tracking.add({
                selector: ((props.selector) ? props.selector : ref.current),
                data: props,
                conversion: props
            });
        });
    }

    /**
     *
     */
    trackPageView() {
        SmartPixelReact.trackPage();
    }

    /**
     *
     */
    /*
    trackAction() {
        webtrekkSmartPixelReact.trackAction();
    }
    */

    /**
     * @override
     */
    componentDidMount() {
        if (this.type === 'init' || this.type === 'advanced') {
            this.addConfigurationData();
        }
        else if (this.type === 'page') {
            this.addPageTrackingData();
        }
        else if (this.type === 'customer') {
            this.addCustomerTrackingData();
        }
        else if (this.type === 'product') {
            this.addProductTrackingData();
        }
        else {
            this.addTrackingData();
        }
    }

    /**
     * @override
     */
    componentDidUpdate() {
        this.componentDidMount();
    }

    /**
     * @override
     *
     * @returns {null}
     */
    render() {
        return null;
    }
}

class WebtrekkAutoTracking extends WebtrekkReactComponent {
    constructor() {
        super('auto');
    }

    componentDidMount() {
        this.addConfigurationData('init');

        if (this.props.activateActions) {
            this.addExtension('action');
        }

        if (this.props.activateTeaser) {
            this.addExtension('teaser_tracking');
        }

        if (this.props.activateProductList) {
            this.addExtension('product_list_tracking');
        }

        if (this.props.activateContentEngagement) {
            this.addExtension('content_engagement');
        }

        this.trackPageView();
    }

    componentDidUpdate({ history }) {
        if (history.action === 'PUSH' || history.action === 'POP') {
            window.setTimeout(() => {
                this.trackPageView();

                if (this.props.activateActions) {
                    this.addExtension('action', 'reload');
                }
            }, 500);
        }
    }
}

WebtrekkAutoTracking.propTypes = {
    trackId: PropTypes.string.isRequired,
    trackDomain: PropTypes.string.isRequired,

    activateActions: PropTypes.bool,
    activateTeaser: PropTypes.bool,
    activateProductList: PropTypes.bool,
    activateContentEngagement: PropTypes.bool
};

WebtrekkAutoTracking.defaultProps = {
    trackId: null,
    trackDomain: null,
    activateActions: false,
    activateTeaser: false,
    activateProductList: false,
    activateContentEngagement: false
};

var AutoTracking = withRouter(WebtrekkAutoTracking);

class WebtrekkInitData extends WebtrekkReactComponent {
    constructor() {
        super('init');
    }
}

WebtrekkInitData.propTypes = {
    trackId: PropTypes.string.isRequired,
    trackDomain: PropTypes.string.isRequired,
    domain: PropTypes.arrayOf(PropTypes.string),
    cookie: PropTypes.oneOf(['1', '3'])
};

WebtrekkInitData.defaultProps = {
    trackId: null,
    trackDomain: null,
    domain: null,
    cookie: null
};

class WebtrekkAdvancedData extends WebtrekkReactComponent {
    constructor() {
        super('advanced');
    }
}

WebtrekkAdvancedData.propTypes = {
    secureCookie: PropTypes.bool,
    optOutName: PropTypes.string,
    requestObfuscation: PropTypes.bool,
    execCDB: PropTypes.bool,
    useCDBCache: PropTypes.bool,
    useHashForDefaultPageName: PropTypes.bool,
    useParamsForDefaultPageName: PropTypes.arrayOf(PropTypes.string),
    requestQueue: PropTypes.shape({
        activated: PropTypes.bool,
        ttl: PropTypes.number,
        resendInterval: PropTypes.number,
        size: PropTypes.number
    })
};

WebtrekkAdvancedData.defaultProps = {
    secureCookie: null,
    optOutName: null,
    requestObfuscation: null,
    execCDB: null,
    useCDBCache: null,
    useHashForDefaultPageName: null,
    useParamsForDefaultPageName: null,
    requestQueue: {
        activated: null,
        ttl: null,
        resendInterval: null,
        size: null
    }
};

class WebtrekkCampaignData extends WebtrekkReactComponent {
    constructor() {
        super('campaign');
    }
}

WebtrekkCampaignData.propTypes = {
    id: PropTypes.string.isRequired,
    mediaCode: PropTypes.arrayOf(PropTypes.string),
    oncePerSession: PropTypes.bool,
    parameter: PropTypes.objectOf(PropTypes.string)
};

WebtrekkCampaignData.defaultProps = {
    id: null,
    mediaCode: null,
    oncePerSession: null,
    parameter: null
};

class WebtrekkCustomerData extends WebtrekkReactComponent {
    constructor() {
        super('customer');
    }
}

WebtrekkCustomerData.propTypes = {
    id: PropTypes.string.isRequired,
    email: PropTypes.string,
    emailRID: PropTypes.string,
    emailOptin: PropTypes.bool,
    firstName: PropTypes.string,
    lastName: PropTypes.string,
    telephone: PropTypes.string,
    gender: PropTypes.number,
    birthday: PropTypes.string,
    country: PropTypes.string,
    city: PropTypes.string,
    postalCode: PropTypes.string,
    street: PropTypes.string,
    streetNumber: PropTypes.string,
    validation: PropTypes.bool,
    category: PropTypes.objectOf(PropTypes.string)
};

WebtrekkCustomerData.defaultProps = {
    id: null,
    email: null,
    emailRID: null,
    emailOptin: null,
    firstName: null,
    lastName: null,
    telephone: null,
    gender: null,
    birthday: null,
    country: null,
    city: null,
    postalCode: null,
    street: null,
    streetNumber: null,
    validation: null,
    category: null
};

class WebtrekkOrderData extends WebtrekkReactComponent {
    constructor() {
        super('order');
    }
}

WebtrekkOrderData.propTypes = {
    value: PropTypes.number.isRequired,
    id: PropTypes.string,
    currency: PropTypes.string,
    couponValue: PropTypes.number,
    paymentMethod: PropTypes.string,
    shippingService: PropTypes.string,
    shippingSpeed: PropTypes.string,
    shippingCosts: PropTypes.number,
    grossMargin: PropTypes.number,
    orderStatus: PropTypes.string,
    parameter: PropTypes.objectOf(PropTypes.string)
};

WebtrekkOrderData.defaultProps = {
    value: null,
    id: null,
    currency: null,
    couponValue: null,
    paymentMethod: null,
    shippingService: null,
    shippingSpeed: null,
    shippingCosts: null,
    grossMargin: null,
    orderStatus: null,
    parameter: null
};

class WebtrekkPageData extends WebtrekkReactComponent {
    constructor() {
        super('page');
    }
}

WebtrekkPageData.propTypes = {
    name: PropTypes.string,
    search: PropTypes.string,
    numberSearchResults: PropTypes.number,
    errorMessages: PropTypes.string,
    paywall: PropTypes.bool,
    articleTitle: PropTypes.string,
    contentTags: PropTypes.string,
    title: PropTypes.string,
    type: PropTypes.string,
    length: PropTypes.string,
    daysSincePublication: PropTypes.number,
    testVariant: PropTypes.string,
    testExperiment: PropTypes.string,
    parameter: PropTypes.objectOf(PropTypes.string),
    category: PropTypes.objectOf(PropTypes.string),
    goal: PropTypes.objectOf(PropTypes.string)
};

WebtrekkPageData.defaultProps = {
    name: null,
    search: null,
    numberSearchResults: null,
    errorMessages: null,
    paywall: null,
    articleTitle: null,
    contentTags: null,
    title: null,
    type: null,
    length: null,
    daysSincePublication: null,
    testVariant: null,
    testExperiment: null,
    parameter: null,
    category: null,
    goal: null
};

class WebtrekkProductData extends WebtrekkReactComponent {
    constructor() {
        super('product');
    }
}

WebtrekkProductData.propTypes = {
    id: PropTypes.string.isRequired,
    action: PropTypes.oneOf(['list', 'view', 'basket', 'confirmation']).isRequired,
    cost: PropTypes.number,
    quantity: PropTypes.number,
    variant: PropTypes.string,
    soldOut: PropTypes.bool,
    parameter: PropTypes.objectOf(PropTypes.string),
    category: PropTypes.objectOf(PropTypes.string)
};

WebtrekkProductData.defaultProps = {
    id: null,
    action: null,
    cost: null,
    quantity: null,
    variant: null,
    soldOut: null,
    parameter: null,
    category: null
};

class WebtrekkSessionData extends WebtrekkReactComponent {
    constructor() {
        super('session');
    }
}

WebtrekkSessionData.propTypes = {
    loginStatus: PropTypes.string,
    parameter: PropTypes.objectOf(PropTypes.string)
};

WebtrekkSessionData.defaultProps = {
    loginStatus: null,
    parameter: null
};

// polyfill for react v15
const createRef = React.createRef || function() {
    function getRef(refObject) {
        if (!refObject) {
            return null;
        }

        let ref = refObject;

        if (Object.keys(ref).length === 1) {
            if (ref.hasOwnProperty('current')) {
                ref = ref.current;
            }
            else if (ref.hasOwnProperty('value')) {
                ref = ref.value;
            }
        }

        return ref;
    }

    function ref(instanceOrNode) {
        ref.current = getRef(instanceOrNode) || null;
    }

    ref.current = null;

    return ref;
};

class WebtrekkTeaser extends WebtrekkReactComponent {
    constructor() {
        super('teaser_tracking');
        this.elementRef = createRef();
    }

    componentDidMount() {
        this.addTeaserElement();
    }

    componentDidUpdate() {
        this.componentDidMount();
    }

    render() {
        const children = React.Children.map(this.props.children, (element) => {
            return React.cloneElement(element, {
                ref: this.elementRef
            });
        });

        return children[0];
    }
}

WebtrekkTeaser.propTypes = {
    selector: PropTypes.string,
    name: PropTypes.string.isRequired,
    rank: PropTypes.string,
    content: PropTypes.string,
    variant: PropTypes.string,
    seen: PropTypes.bool,
    type: PropTypes.oneOf(['view', 'click', 'product']),
    goal: PropTypes.oneOf(['order', 'goal', 'both']),
    value: PropTypes.string
};

WebtrekkTeaser.defaultProps = {
    selector: null,
    name: null,
    rank: null,
    content: null,
    variant: null,
    seen: null,
    type: null,
    goal: null,
    value: null
};

class WebtrekkProductList extends WebtrekkReactComponent {
    constructor() {
        super('product_list_tracking');
        this.elementRef = createRef();
    }

    componentDidMount() {
        this.addProductListElement();
    }

    componentDidUpdate() {
        this.componentDidMount();
    }

    render() {
        const children = React.Children.map(this.props.children, (element) => {
            return React.cloneElement(element, {
                ref: this.elementRef
            });
        });

        return children[0];
    }
}

WebtrekkProductList.propTypes = {
    selector: PropTypes.string,
    id: PropTypes.string.isRequired,
    position: PropTypes.number.isRequired,
    cost: PropTypes.number,
    quantity: PropTypes.number,
    variant: PropTypes.string,
    soldOut: PropTypes.bool,
    category: PropTypes.objectOf(PropTypes.string),
    parameter: PropTypes.objectOf(PropTypes.string)
};

WebtrekkProductList.defaultProps = {
    selector: null,
    id: null,
    position: null,
    cost: null,
    quantity: null,
    variant: null,
    soldOut: null,
    category: null,
    parameter: null
};

class WebtrekkContentEngagement extends WebtrekkReactComponent {
    constructor() {
        super('content_engagement');
        this.elementRef = createRef();
    }

    componentDidMount() {
        this.addContentEngagementElement();
    }

    componentDidUpdate() {
        this.componentDidMount();
    }

    render() {
        const children = React.Children.map(this.props.children, (element) => {
            return React.cloneElement(element, {
                ref: this.elementRef
            });
        });

        return children[0];
    }
}

WebtrekkContentEngagement.propTypes = {
    selector: PropTypes.string,
    name: PropTypes.string,
    percentageStepsInAnalytics: PropTypes.number,
    sendContentEngagement: PropTypes.oneOf([0, 1, 2]),
    percentageReached: PropTypes.number,
    secondsReached: PropTypes.number,
    largeBrowserResolution: PropTypes.number,
    largeBrowserSeconds: PropTypes.number,
    mediumBrowserResolution: PropTypes.number,
    mediumBrowserSeconds: PropTypes.number,
    smallBrowserResolution: PropTypes.number,
    smallBrowserSeconds: PropTypes.number
};

WebtrekkContentEngagement.defaultProps = {
    selector: null,
    name: null,
    percentageStepsInAnalytics: null,
    sendContentEngagement: null,
    percentageReached: null,
    secondsReached: null,
    largeBrowserResolution: null,
    largeBrowserSeconds: null,
    mediumBrowserResolution: null,
    mediumBrowserSeconds: null,
    smallBrowserResolution: null,
    smallBrowserSeconds: null
};

class WebtrekkExtension extends WebtrekkReactComponent {
    constructor() {
        super('extension');
    }

    componentDidMount() {
        this.addExtension(this.props.name, this.props.action, this.props.config);
    }
}

WebtrekkExtension.propTypes = {
    name: PropTypes.string.isRequired,
    action: PropTypes.string.isRequired,
    config: PropTypes.object
};

WebtrekkExtension.defaultProps = {
    name: null,
    action: null,
    config: null
};

const webtrekkMiddleware = (actions = {}) => ({ dispatch, getState }) => next => action => {
    var nextValue = next(action);

    if (typeof actions[action.type] === 'function') {
        actions[action.type](getState(), action);
        return nextValue;
    }

    if (!action.webtrekk) {
        return nextValue;
    }

    var isWebtrekkAction = true;
    var webtrekk = action.webtrekk;
    var type = webtrekk.type || '';
    var data = webtrekk.data || {};
    var sendInstantly = webtrekk.sendInstantly || false;

    switch (type) {
        case 'webtrekk/init': SmartPixelReact.init(data); break;
        case 'webtrekk/advanced': SmartPixelReact.advanced(data); break;
        case 'webtrekk/page': SmartPixelReact.page(data.name, data); break;
        case 'webtrekk/action': SmartPixelReact.action(data); break;
        case 'webtrekk/session': SmartPixelReact.session(data); break;
        case 'webtrekk/campaign': SmartPixelReact.campaign(data); break;
        case 'webtrekk/customer': SmartPixelReact.customer(data.id, data, data.validation); break;
        case 'webtrekk/product': SmartPixelReact.product(webtrekk.action, data); break;
        case 'webtrekk/products': SmartPixelReact.products(webtrekk.action, data); break;
        case 'webtrekk/order': SmartPixelReact.order(data); break;
        case 'webtrekk/extension': SmartPixelReact.extension(data.extension, data.action, data); break;
        case 'webtrekk/track': SmartPixelReact.track(data); break;
        case 'webtrekk/trackPage': SmartPixelReact.trackPage(data); break;
        case 'webtrekk/trackAction': SmartPixelReact.trackAction(data); break;
        default: isWebtrekkAction = false; break;
    }

    if (isWebtrekkAction && sendInstantly) {
        SmartPixelReact.track();
    }

    return nextValue;
};

const webtrekkReducer = (actions = {}) => (customReducer = () => {}) => useCallback((state, action) => {
    var customReducerValue = customReducer(state, action);

    if (typeof actions[action.type] === 'function') {
        actions[action.type](state, action, customReducerValue);
        return customReducerValue;
    }

    if (!action.webtrekk) {
        return customReducerValue;
    }

    var isWebtrekkAction = true;
    var webtrekk = action.webtrekk;
    var type = webtrekk.type || '';
    var data = webtrekk.data || {};
    var sendInstantly = webtrekk.sendInstantly || false;

    switch (type) {
        case 'webtrekk/init': SmartPixelReact.init(data); break;
        case 'webtrekk/advanced': SmartPixelReact.advanced(data); break;
        case 'webtrekk/page': SmartPixelReact.page(data.name, data); break;
        case 'webtrekk/action': SmartPixelReact.action(data); break;
        case 'webtrekk/session': SmartPixelReact.session(data); break;
        case 'webtrekk/campaign': SmartPixelReact.campaign(data); break;
        case 'webtrekk/customer': SmartPixelReact.customer(data.id, data, data.validation); break;
        case 'webtrekk/product': SmartPixelReact.product(webtrekk.action, data); break;
        case 'webtrekk/products': SmartPixelReact.products(webtrekk.action, data); break;
        case 'webtrekk/order': SmartPixelReact.order(data); break;
        case 'webtrekk/extension': SmartPixelReact.extension(data.extension, data.action, data); break;
        case 'webtrekk/track': SmartPixelReact.track(data); break;
        case 'webtrekk/trackPage': SmartPixelReact.trackPage(data); break;
        case 'webtrekk/trackAction': SmartPixelReact.trackAction(data); break;
        default: isWebtrekkAction = false; break;
    }

    if (isWebtrekkAction && sendInstantly) {
        SmartPixelReact.track();
    }

    return customReducerValue;

}, [customReducer, actions]);

const WebtrekkAutoTracking$1 = AutoTracking;
const WebtrekkInitData$1 = WebtrekkInitData;
const WebtrekkAdvancedData$1 = WebtrekkAdvancedData;
const WebtrekkCampaignData$1 = WebtrekkCampaignData;
const WebtrekkCustomerData$1 = WebtrekkCustomerData;
const WebtrekkOrderData$1 = WebtrekkOrderData;
const WebtrekkPageData$1 = WebtrekkPageData;
const WebtrekkProductData$1 = WebtrekkProductData;
const WebtrekkSessionData$1 = WebtrekkSessionData;
const WebtrekkTeaser$1 = WebtrekkTeaser;
const WebtrekkProductList$1 = WebtrekkProductList;
const WebtrekkContentEngagement$1 = WebtrekkContentEngagement;
const WebtrekkExtension$1 = WebtrekkExtension;
const WebtrekkSmartPixelReact$1 = SmartPixelReact;
const webtrekkMiddleware$1 = webtrekkMiddleware;
const webtrekkReducer$1 = webtrekkReducer;

// compatibility for v0
const webtrekkSmartPixelReact = SmartPixelReact;

var index = {
    WebtrekkAutoTracking: WebtrekkAutoTracking$1,
    WebtrekkInitData: WebtrekkInitData$1,
    WebtrekkAdvancedData: WebtrekkAdvancedData$1,
    WebtrekkCampaignData: WebtrekkCampaignData$1,
    WebtrekkCustomerData: WebtrekkCustomerData$1,
    WebtrekkOrderData: WebtrekkOrderData$1,
    WebtrekkPageData: WebtrekkPageData$1,
    WebtrekkProductData: WebtrekkProductData$1,
    WebtrekkSessionData: WebtrekkSessionData$1,
    WebtrekkTeaser: WebtrekkTeaser$1,
    WebtrekkProductList: WebtrekkProductList$1,
    WebtrekkContentEngagement: WebtrekkContentEngagement$1,
    WebtrekkExtension: WebtrekkExtension$1,
    WebtrekkSmartPixelReact: WebtrekkSmartPixelReact$1,
	webtrekkMiddleware: webtrekkMiddleware$1,
    webtrekkReducer: webtrekkReducer$1,

    // compatibility for v0
    webtrekkSmartPixelReact
};

export { WebtrekkAutoTracking$1 as WebtrekkAutoTracking, WebtrekkInitData$1 as WebtrekkInitData, WebtrekkAdvancedData$1 as WebtrekkAdvancedData, WebtrekkCampaignData$1 as WebtrekkCampaignData, WebtrekkCustomerData$1 as WebtrekkCustomerData, WebtrekkOrderData$1 as WebtrekkOrderData, WebtrekkPageData$1 as WebtrekkPageData, WebtrekkProductData$1 as WebtrekkProductData, WebtrekkSessionData$1 as WebtrekkSessionData, WebtrekkTeaser$1 as WebtrekkTeaser, WebtrekkProductList$1 as WebtrekkProductList, WebtrekkContentEngagement$1 as WebtrekkContentEngagement, WebtrekkExtension$1 as WebtrekkExtension, WebtrekkSmartPixelReact$1 as WebtrekkSmartPixelReact, webtrekkMiddleware$1 as webtrekkMiddleware, webtrekkReducer$1 as webtrekkReducer, webtrekkSmartPixelReact };
export default index;
